@@ -1409,10 +1409,27 @@ class ChatLoop:
                                         )
                                         print(formatted_text, end="", flush=True)
             else:
-                # Fallback to synchronous call if streaming not supported
-                response = await asyncio.get_event_loop().run_in_executor(
-                    None, self.agent, query
-                )
+                # Fallback to non-streaming call if streaming not supported
+                # Check if agent has async run method (Google ADK agents)
+                if hasattr(self.agent, "run_async"):
+                    response = await self.agent.run_async(query)
+                elif callable(self.agent):
+                    # Standard callable agents
+                    response = await asyncio.get_event_loop().run_in_executor(
+                        None, self.agent, query
+                    )
+                else:
+                    # Try run_live as fallback for Google ADK
+                    if hasattr(self.agent, "run_live"):
+                        # run_live is synchronous, so use executor
+                        response = await asyncio.get_event_loop().run_in_executor(
+                            None, self.agent.run_live, query
+                        )
+                    else:
+                        raise TypeError(
+                            f"Agent of type {type(self.agent).__name__} is not "
+                            "callable and does not have run_async or run_live methods"
+                        )
                 response_obj = response  # Store for token extraction
 
                 # Log response received from agent
@@ -1426,7 +1443,11 @@ class ChatLoop:
                     await thinking_task
 
                 # Format and display response
-                if hasattr(response, "message"):
+                # Check for Google ADK response format first
+                if hasattr(response, "text"):
+                    # Google ADK returns a response with .text attribute
+                    response_text.append(response.text)
+                elif hasattr(response, "message"):
                     message = response.message
                     if isinstance(message, dict) and "content" in message:
                         content = message["content"]
@@ -1438,6 +1459,8 @@ class ChatLoop:
                             response_text.append(str(content))
                     else:
                         response_text.append(str(message))
+                elif isinstance(response, str):
+                    response_text.append(response)
                 else:
                     response_text.append(str(response))
 
